[PATCH] D78235: [clangd] Store ppdirective in Inclusion

Kadir Cetinkaya via Phabricator via cfe-commits cfe-commits at lists.llvm.org
Tue Apr 21 01:35:36 PDT 2020


This revision was automatically updated to reflect the committed changes.
Closed by commit rG6e017188b760: [clangd] Store ppdirective in Inclusion (authored by kadircet).

Changed prior to commit:
  https://reviews.llvm.org/D78235?vs=257812&id=258926#toc

Repository:
  rG LLVM Github Monorepo

CHANGES SINCE LAST ACTION
  https://reviews.llvm.org/D78235/new/

https://reviews.llvm.org/D78235

Files:
  clang-tools-extra/clangd/Headers.cpp
  clang-tools-extra/clangd/Headers.h
  clang-tools-extra/clangd/unittests/HeadersTests.cpp


Index: clang-tools-extra/clangd/unittests/HeadersTests.cpp
===================================================================
--- clang-tools-extra/clangd/unittests/HeadersTests.cpp
+++ clang-tools-extra/clangd/unittests/HeadersTests.cpp
@@ -11,6 +11,7 @@
 #include "Compiler.h"
 #include "TestFS.h"
 #include "TestTU.h"
+#include "clang/Basic/TokenKinds.h"
 #include "clang/Frontend/CompilerInstance.h"
 #include "clang/Frontend/CompilerInvocation.h"
 #include "clang/Frontend/FrontendActions.h"
@@ -127,6 +128,7 @@
 MATCHER_P(Written, Name, "") { return arg.Written == Name; }
 MATCHER_P(Resolved, Name, "") { return arg.Resolved == Name; }
 MATCHER_P(IncludeLine, N, "") { return arg.R.start.line == N; }
+MATCHER_P(Directive, D, "") { return arg.Directive == D; }
 
 MATCHER_P2(Distance, File, D, "") {
   if (arg.getKey() != File)
@@ -201,6 +203,19 @@
               UnorderedElementsAre(Distance(MainFile, 0u)));
 }
 
+TEST_F(HeadersTest, IncludeDirective) {
+  FS.Files[MainFile] = R"cpp(
+#include "foo.h"
+#import "foo.h"
+#include_next "foo.h"
+)cpp";
+
+  EXPECT_THAT(collectIncludes().MainFileIncludes,
+              UnorderedElementsAre(Directive(tok::pp_include),
+                                   Directive(tok::pp_import),
+                                   Directive(tok::pp_include_next)));
+}
+
 TEST_F(HeadersTest, InsertInclude) {
   std::string Path = testPath("sub/bar.h");
   FS.Files[Path] = "";
Index: clang-tools-extra/clangd/Headers.cpp
===================================================================
--- clang-tools-extra/clangd/Headers.cpp
+++ clang-tools-extra/clangd/Headers.cpp
@@ -28,7 +28,7 @@
 
   // Record existing #includes - both written and resolved paths. Only #includes
   // in the main file are collected.
-  void InclusionDirective(SourceLocation HashLoc, const Token & /*IncludeTok*/,
+  void InclusionDirective(SourceLocation HashLoc, const Token &IncludeTok,
                           llvm::StringRef FileName, bool IsAngled,
                           CharSourceRange FilenameRange, const FileEntry *File,
                           llvm::StringRef /*SearchPath*/,
@@ -44,6 +44,7 @@
       Inc.Resolved = std::string(File ? File->tryGetRealPathName() : "");
       Inc.HashOffset = SM.getFileOffset(HashLoc);
       Inc.FileKind = FileKind;
+      Inc.Directive = IncludeTok.getIdentifierInfo()->getPPKeywordID();
     }
     if (File) {
       auto *IncludingFileEntry = SM.getFileEntryForID(SM.getFileID(HashLoc));


-------------- next part --------------
A non-text attachment was scrubbed...
Name: D78235.258926.patch
Type: text/x-patch
Size: 3797 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/cfe-commits/attachments/20200421/c60e10a9/attachment-0001.bin>


More information about the cfe-commits mailing list
Index: clang-tools-extra/clangd/Headers.h
===================================================================
--- clang-tools-extra/clangd/Headers.h
+++ clang-tools-extra/clangd/Headers.h
@@ -13,6 +13,7 @@
 #include "Protocol.h"
 #include "SourceCode.h"
 #include "index/Symbol.h"
+#include "clang/Basic/TokenKinds.h"
 #include "clang/Format/Format.h"
 #include "clang/Lex/HeaderSearch.h"
 #include "clang/Lex/PPCallbacks.h"
@@ -22,6 +23,7 @@
 #include "llvm/ADT/StringSet.h"
 #include "llvm/Support/Error.h"
 #include "llvm/Support/VirtualFileSystem.h"
+#include <string>
 
 namespace clang {
 namespace clangd {
@@ -50,9 +52,10 @@
 
 // An #include directive that we found in the main file.
 struct Inclusion {
-  Range R;             // Inclusion range.
-  std::string Written; // Inclusion name as written e.g. <vector>.
-  Path Resolved;       // Resolved path of included file. Empty if not resolved.
+  Range R;                      // Inclusion range.
+  tok::PPKeywordKind Directive; // Directive used for inclusion, e.g. import
+  std::string Written;          // Inclusion name as written e.g. <vector>.
+  Path Resolved; // Resolved path of included file. Empty if not resolved.
   unsigned HashOffset = 0; // Byte offset from start of file to #.
   SrcMgr::CharacteristicKind FileKind = SrcMgr::C_User;
 };